@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: Widget.c,v 1.125 2003/06/05 18:47:00 dk Exp $
+ * $Id: Widget.c,v 1.129 2003/08/27 18:59:24 dk Exp $
  */
 
 #include "apricot.h"
@@ -60,12 +60,15 @@ static Bool find_dup_msg( PEvent event, int * cmd);
 static Bool pquery ( Handle window, Handle self, void * v);
 static Bool get_top_current( Handle self);
 static Bool sptr( Handle window, Handle self, void * v);
-static Bool size_notify( Handle self, Handle child, const Rect* metrix);
-static Bool move_notify( Handle self, Handle child, Point * moveTo);
 static Handle find_tabfoc( Handle self);
 static Bool showhint_notify ( Handle self, Handle child, void * data);
 static Bool hint_notify ( Handle self, Handle child, SV * hint);
 
+extern void Widget_pack_slaves( Handle self); 
+extern void Widget_place_slaves( Handle self); 
+extern Bool Widget_size_notify( Handle self, Handle child, const Rect* metrix);
+extern Bool Widget_move_notify( Handle self, Handle child, Point * moveTo);
+
 /* init, done & update_sys_handle */
 void
 Widget_init( Handle self, HV * profile)
@@ -78,6 +81,9 @@ Widget_init( Handle self, HV * profile)
    list_create( &var-> widgets, 0, 8);
    var-> tabOrder = -1;
 
+   var-> geomInfo. side = 3; /* default pack side is 'top', anchor ='center' */
+   var-> geomInfo. anchorx = var-> geomInfo. anchory = 1;
+
    my-> update_sys_handle( self, profile);
    /* props init */
    /* font and colors */
@@ -187,6 +193,7 @@ Widget_init( Handle self, HV * profile)
       my-> set_size( self, size);
    } else
       var-> virtualSize = my-> get_size( self);
+   var-> geomSize = var-> virtualSize;
 
    {
       Bool x = 0, y = 0;
@@ -195,6 +202,12 @@ Widget_init( Handle self, HV * profile)
       if ( pget_B( y_centered) || ( var-> growMode & gmYCenter)) y = 1;
       if ( x || y) my-> set_centered( self, x, y);
    }
+
+   opt_assign( optPackPropagate, pget_B( packPropagate));
+   my-> set_packInfo( self, pget_sv( packInfo));
+   my-> set_placeInfo( self, pget_sv( placeInfo));
+   my-> set_geometry( self, pget_i( geometry));
+   
    my-> set_shape       ( self, pget_H(  shape));
    my-> set_visible     ( self, pget_B( visible));
    if ( pget_B( capture)) my-> set_capture( self, 1, nilHandle);
@@ -315,7 +328,25 @@ Widget_can_close( Handle self)
 void
 Widget_cleanup( Handle self)
 {
+   Handle ptr;
    enter_method;
+
+   /* disconnect all geometry slaves */
+   ptr = var-> packSlaves;
+   while ( ptr) {
+      PWidget( ptr)-> geometry = gtDefault;
+      ptr = PWidget( ptr)-> geomInfo. next;
+   }
+   var-> packSlaves = nilHandle;
+   ptr = var-> placeSlaves;
+   while ( ptr) {
+      PWidget( ptr)-> geometry = gtDefault;
+      ptr = PWidget( ptr)-> geomInfo. next;
+   }
+   var-> placeSlaves = nilHandle;
+   
+   my-> set_geometry( self, gtDefault);
+   
    if ( application && (( PApplication) application)-> hintUnder == self)
       my-> set_hintVisible( self, 0);
 
@@ -763,7 +794,7 @@ void Widget_handle_event( Handle self, PEvent event)
             }
           MOVE_EVENT:;
             if ( !event-> gen. B)
-               my-> first_that( self, (void*)move_notify, &event-> gen. P);
+               my-> first_that( self, (void*) Widget_move_notify, &event-> gen. P);
             if ( doNotify) oldP = var-> pos;
             var-> pos = event-> gen. P;
             if ( doNotify && 
@@ -771,7 +802,8 @@ void Widget_handle_event( Handle self, PEvent event)
                   oldP. y != event-> gen. P. y)) {
                my-> notify( self, "<sPP", "Move", oldP, event-> gen. P);
                objCheck;
-               if ( var-> growMode & gmCenter) my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
+               if ( var-> growMode & gmCenter) 
+                  my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
             }
          }
         break;
@@ -819,15 +851,18 @@ void Widget_handle_event( Handle self, PEvent event)
               n-> gen. P. y = n-> gen. R. top    = event-> gen. P. y;
            }
         SIZE_EVENT:;  
-           if ( var-> growMode & gmCenter) my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
-
-           if ( !event-> gen. B) my-> first_that( self, (void*)size_notify, &event-> gen. R);
+           if ( var-> growMode & gmCenter) 
+               my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
+           if ( !event-> gen. B)
+               my-> first_that( self, (void*) Widget_size_notify, &event-> gen. R);
            if ( doNotify) {
               Point oldSize;
               oldSize. x = event-> gen. R. left;
               oldSize. y = event-> gen. R. bottom;
               my-> notify( self, "<sPP", "Size", oldSize, event-> gen. P);
            }
+           Widget_pack_slaves( self);
+           Widget_place_slaves( self);
         }
         break;
    }
@@ -1152,6 +1187,8 @@ Widget_next_tab( Handle self, Bool forward)
 
 /*::o */
 /*::p */
+
+
 void
 Widget_post_message( Handle self, SV * info1, SV * info2)
 {
@@ -1264,8 +1301,9 @@ void
 Widget_set( Handle self, HV * profile)
 {
    enter_method;
-   Handle postOwner = var-> owner;
+   Handle postOwner = nilHandle;
    AV *order = nil;
+   int geometry = gtDefault;
 
    if ( pexist(__ORDER__)) order = (AV*)SvRV(pget_sv( __ORDER__));
 
@@ -1296,6 +1334,10 @@ Widget_set( Handle self, HV * profile)
             opt_set( optOwnerFont);
          }
       }
+      if ( var-> geometry != gtDefault) {
+         geometry = var-> geometry;
+         my-> set_geometry( self, gtDefault);
+      }
    }
 
    /* geometry manipulations */
@@ -1488,14 +1530,19 @@ Widget_set( Handle self, HV * profile)
       my-> set_cursorPos( self, set);
       pdelete( cursorPos);
    }
+   if ( pexist( geomSize))
+   {
+      Point set;
+      prima_read_point( pget_sv( geomSize), (int*)&set, 2, "RTC0089: Array panic on 'geomSize'");
+      my-> set_geomSize( self, set);
+      pdelete( geomSize);
+   }
 
    inherited-> set( self, profile);
-   if ( var-> owner != postOwner)
-   {
-       var-> owner = postOwner;
-       my-> set_tabOrder( self, var-> tabOrder);
+   if ( postOwner) {
+      my-> set_tabOrder( self, var-> tabOrder);
+      my-> set_geometry( self, geometry);
    }
-   if ( var-> growMode & gmCenter) my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
 }
 
 void
@@ -1621,6 +1668,7 @@ Widget_get_handle( Handle self)
    return newSVpv( buf, 0);
 }
 
+
 SV *
 Widget_get_parent_handle( Handle self)
 {
@@ -1867,67 +1915,6 @@ sptr( Handle window, Handle self, void * v)
 
 /* static iterators for ownership notifications */
 
-static Bool
-size_notify( Handle self, Handle child, const Rect* metrix)
-{
-   if ( his-> growMode) {
-      Point size  =  his-> self-> get_virtual_size( child);
-      Point pos   =  his-> self-> get_origin( child);
-      Point osize = size, opos = pos;
-      int   dx    = ((Rect *) metrix)-> right - ((Rect *) metrix)-> left;
-      int   dy    = ((Rect *) metrix)-> top   - ((Rect *) metrix)-> bottom;
-
-      if ( his-> growMode & gmGrowLoX) pos.  x += dx;
-      if ( his-> growMode & gmGrowHiX) size. x += dx;
-      if ( his-> growMode & gmGrowLoY) pos.  y += dy;
-      if ( his-> growMode & gmGrowHiY) size. y += dy;
-      if ( his-> growMode & gmXCenter) pos. x = (((Rect *) metrix)-> right - size. x) / 2;
-      if ( his-> growMode & gmYCenter) pos. y = (((Rect *) metrix)-> top   - size. y) / 2;
-
-      if ( pos.x != opos.x || pos.y != opos.y || size.x != osize.x || size.y != osize.y) {
-         if ( pos.x == opos.x && pos.y == opos.y) {
-            his-> self-> set_size( child, size);
-         } else if ( size.x == osize.x && size.y == osize.y) {
-            his-> self-> set_origin( child, pos);
-         } else {
-            Rect r;
-            r. left   = pos. x;
-            r. bottom = pos. y;
-            r. right  = pos. x + size. x;
-            r. top    = pos. y + size. y;
-            his-> self-> set_rect( child, r);
-         }
-      }
-   }
-   return false;
-}
-
-static Bool
-move_notify( Handle self, Handle child, Point * moveTo)
-{
-   Bool clp = his-> self-> get_clipOwner( child);
-   int  dx  = moveTo-> x - var-> pos. x;
-   int  dy  = moveTo-> y - var-> pos. y;
-   Point p;
-
-   if ( his-> growMode & gmDontCare) {
-      if ( !clp) return false;
-      p = his-> self-> get_origin( child);
-      p. x -= dx;
-      p. y -= dy;
-      his-> self-> set_origin( child, p);
-   } else {
-      if ( clp) return false;
-      p = his-> self-> get_origin( child);
-      p. x += dx;
-      p. y += dy;
-      his-> self-> set_origin( child, p);
-   }
-
-   return false;
-}
-
-
 Bool
 font_notify ( Handle self, Handle child, void * font)
 {
@@ -2786,44 +2773,6 @@ Widget_size( Handle self, Bool set, Point size)
    return size;
 }
 
-Point
-Widget_sizeMin( Handle self, Bool set, Point min)
-{
-   enter_method;
-   if ( !set)
-      return var-> sizeMin;
-   var-> sizeMin = min;
-   if ( var-> stage <= csFrozen) {
-      Point sizeActual  = my-> get_size( self);
-      Point newSize     = sizeActual;
-      if ( sizeActual. x < min. x) newSize. x = min. x;
-      if ( sizeActual. y < min. y) newSize. y = min. y;
-      if (( newSize. x != sizeActual. x) || ( newSize. y != sizeActual. y))
-         my-> set_size( self, newSize);
-   }
-   apc_widget_set_size_bounds( self, var-> sizeMin, var-> sizeMax);
-   return min;
-}
-
-Point
-Widget_sizeMax( Handle self, Bool set, Point max)
-{
-   enter_method;
-   if ( !set)
-      return var-> sizeMax;
-   var-> sizeMax = max;
-   if ( var-> stage <= csFrozen) {
-      Point sizeActual  = my-> get_size( self);
-      Point newSize     = sizeActual;
-      if ( sizeActual. x > max. x) newSize. x = max. x;
-      if ( sizeActual. y > max. y) newSize. y = max. y;
-      if (( newSize. x != sizeActual. x) || ( newSize. y !=  sizeActual. y))
-          my-> set_size( self, newSize);
-   }
-   apc_widget_set_size_bounds( self, var-> sizeMin, var-> sizeMax);
-   return max;
-}
-
 Bool
 Widget_syncPaint( Handle self, Bool set, Bool syncPaint)
 {
@@ -2892,7 +2841,7 @@ Widget_tabOrder( Handle self, Bool set, int tabOrder)
           }
        }
        if ( match)
-          /* incrementing all tabOrders that greater than out */
+          /* incrementing all tabOrders that greater than ours */
           for ( i = 0; i < count; i++) {
              PWidget ctrl = ( PWidget) owner-> widgets. items[ i];
              if ( self == ( Handle) ctrl) continue;
