@@ -1,5 +1,5 @@
 #
-#  Copyright (c) 1997-2000 The Protein Laboratory, University of Copenhagen
+#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
 #  All rights reserved.
 #
 #  Redistribution and use in source and binary forms, with or without
@@ -23,6 +23,7 @@
 #  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 #  SUCH DAMAGE.
 #
+#  $Id: eyes.pl,v 1.6 2003/08/01 09:45:30 dk Exp $
 #
 =pod 
 =item NAME
@@ -83,7 +84,7 @@ sub reshape
 
 my $m;
 
-my $x = Prima::Window-> create(
+my $x = Prima::MainWindow-> create(
    visible  => 0,
    buffered  => 1,
    color     => cl::Black,
