@@ -26,16 +26,16 @@
  * Created by Dmitry Karasik <dmitry@karasik.eu.org> with great help
  * of tiff2png.c by Willem van Schaik and Greg Roelofs
  *
- * $Id: codec_tiff.c,v 1.1 2003/03/25 17:55:11 dk Exp $
+ * $Id: codec_tiff.c,v 1.3 2003/06/18 16:40:44 dk Exp $
  */
 
+#include "img.h"
+#include "img_conv.h"
+#include "Icon.h"
 #include <tiff.h>
 #include <tiffio.h>
 #include <tiffconf.h>
 #include <stdarg.h>
-#include "img.h"
-#include "unix/guts.h"
-#include "Icon.h"
 
 #ifdef __cplusplus
 extern "C" {
